#include <qaction.h>
#include <qapplication.h>
#include <qlabel.h>
#include <qmenubar.h>
#include <qmessagebox.h>
#include <qstatusbar.h>
#include <qworkspace.h>
#include <qmessagebox.h>
#include <qprocess.h>
#include <iostream>
using std::cout;

#include "editor.h"
#include "mainwindow.h"
#include "simulator.h"
#include "watchwindow.h"

extern QString curhex;

MainWindow::MainWindow(QWidget *parent, const char *name)
    : QMainWindow(parent, name)
{
    compmess = new QMessageBox( "Compile", "No valid pic C source code...", QMessageBox::Information, QMessageBox::Ok, 0, 0, 0, 0, FALSE );
         upmess = new QMessageBox( "Upload", "No board Connected",  QMessageBox::Information, QMessageBox::Ok, 0, 0, 0, 0, FALSE );
       dellmess = new QMessageBox( "Clean the Board", "There is No Board Connected",  QMessageBox::Information, QMessageBox::Ok, 0, 0, 0, 0, FALSE );   
       debugmess = new QMessageBox( "Debugger Message:", "Debugging Process Couldn't Start",  QMessageBox::Information, QMessageBox::Ok, 0, 0, 0, 0, FALSE ); 
       
    workspace = new QWorkspace(this);
    setCentralWidget(workspace);
    connect(workspace, SIGNAL(windowActivated(QWidget *)),
            this, SLOT(updateMenus()));
    connect(workspace, SIGNAL(windowActivated(QWidget *)),
            this, SLOT(updateModIndicator()));
   
     dir  = new QProcess( this );
     sdcc = new QProcess( this );
     debugprocess = new QProcess(this);
     odysseyrem = new QProcess(this);
     odysseywri   = new QProcess(this);
    createActions();
    createMenus();
    createToolBars();
    createStatusBar();

    setCaption(tr("PicSim"));
    setIcon(QPixmap::fromMimeSource("icon.png")); 

}

void MainWindow::createActions()
{
    newAct = new QAction(tr("&New"), tr("Ctrl+N"), this);
    newAct->setIconSet(QPixmap::fromMimeSource("new.png"));
    newAct->setStatusTip(tr("Create a new file"));
    connect(newAct, SIGNAL(activated()), this, SLOT(newFile()));

    openAct = new QAction(tr("&Open..."), tr("Ctrl+O"), this);
    openAct->setIconSet(QPixmap::fromMimeSource("oopen.png"));
    openAct->setStatusTip(tr("Open an existing file"));
    connect(openAct, SIGNAL(activated()), this, SLOT(open()));

    saveAct = new QAction(tr("&Save"), tr("Ctrl+S"), this);
    saveAct->setIconSet(QPixmap::fromMimeSource("saave.png"));
    saveAct->setStatusTip(tr("Save the file to disk"));
    connect(saveAct, SIGNAL(activated()), this, SLOT(save()));

    saveAsAct = new QAction(tr("Save &As..."), 0, this);
    saveAsAct->setStatusTip(tr("Save the file under a new name"));
    connect(saveAsAct, SIGNAL(activated()), this, SLOT(saveAs()));

    exitAct = new QAction(tr("E&xit"), tr("Ctrl+Q"), this);
    exitAct->setStatusTip(tr("Exit the application"));
    connect(exitAct, SIGNAL(activated()), this, SLOT(close()));

    cutAct = new QAction(tr("Cu&t"), tr("Ctrl+X"), this);
    cutAct->setIconSet(QPixmap::fromMimeSource("cut.png"));
    cutAct->setStatusTip(tr("Cut the current selection to the "
                            "clipboard"));
    connect(cutAct, SIGNAL(activated()), this, SLOT(cut()));

    copyAct = new QAction(tr("&Copy"), tr("Ctrl+C"), this);
    copyAct->setIconSet(QPixmap::fromMimeSource("copy.png"));
    copyAct->setStatusTip(tr("Copy the current selection to the "
                             "clipboard"));
    connect(copyAct, SIGNAL(activated()), this, SLOT(copy()));

    pasteAct = new QAction(tr("&Paste"), tr("Ctrl+V"), this);
    pasteAct->setIconSet(QPixmap::fromMimeSource("paste.png"));
    pasteAct->setStatusTip(tr("Paste the clipboard's contents at "
                              "the cursor position"));
    connect(pasteAct, SIGNAL(activated()), this, SLOT(paste()));

    deleteAct = new QAction(tr("&Delete"), tr("Del"), this);
    deleteAct->setIconSet(QPixmap::fromMimeSource("delete.png"));
    deleteAct->setStatusTip(tr("Delete the current selection"));
    connect(deleteAct, SIGNAL(activated()), this, SLOT(del()));

    closeAct = new QAction(tr("Cl&ose"), tr("Ctrl+F4"), this);
    closeAct->setStatusTip(tr("Close the active window"));
    connect(closeAct, SIGNAL(activated()),
            workspace, SLOT(closeActiveWindow()));

    closeAllAct = new QAction(tr("Close &All"), 0, this);
    closeAllAct->setStatusTip(tr("Close all the windows"));
    connect(closeAllAct, SIGNAL(activated()),
            workspace, SLOT(closeAllWindows()));

    tileAct = new QAction(tr("&Tile"), 0, this);
    tileAct->setStatusTip(tr("Tile the windows"));
    connect(tileAct, SIGNAL(activated()), workspace, SLOT(tile()));

    cascadeAct = new QAction(tr("&Cascade"), 0, this);
    cascadeAct->setStatusTip(tr("Cascade the windows"));
    connect(cascadeAct, SIGNAL(activated()),
            workspace, SLOT(cascade()));

    nextAct = new QAction(tr("Ne&xt"), tr("Ctrl+F6"), this);
    nextAct->setStatusTip(tr("Move the focus to the next window"));
    connect(nextAct, SIGNAL(activated()),
            workspace, SLOT(activateNextWindow()));

    previousAct = new QAction(tr("Pre&vious"),
                              tr("Ctrl+Shift+F6"), this);
    previousAct->setStatusTip(tr("Move the focus to the previous "
                                 "window"));
    connect(previousAct, SIGNAL(activated()),
            workspace, SLOT(activatePreviousWindow()));

    aboutAct = new QAction(tr("&About"), 0, this);
    aboutAct->setStatusTip(tr("Show the application's About box"));
    connect(aboutAct, SIGNAL(activated()), this, SLOT(about()));

    aboutQtAct = new QAction(tr("About &Qt"), 0, this);
    aboutQtAct->setStatusTip(tr("Show the Qt library's About box"));
    connect(aboutQtAct, SIGNAL(activated()), qApp, SLOT(aboutQt()));
    
    compileAct= new QAction(tr("&Compile"),0,this);	
    compileAct->setStatusTip(tr("Compile current project"));
    connect(compileAct, SIGNAL(activated()), this, SLOT(compileWindowOpen())); 
    
    debugAct= new QAction(tr("&Debug"),0,this);	
    debugAct->setStatusTip(tr("Debug current project"));
    connect(debugAct, SIGNAL(activated()), this, SLOT(debugWindowOpen())); 
    
    simulateAct= new QAction(tr("&Start Simulation"),0,this);	
    simulateAct->setStatusTip(tr("Simulate current project"));
    connect(simulateAct, SIGNAL(activated()), this, SLOT(simulateWindowOpen())); 
    
    uploadAct=new QAction(tr("&Upload to the Board"), 0, this);
    uploadAct->setStatusTip(tr("Upload current hex file to the board"));
    connect(uploadAct, SIGNAL(activated()), this, SLOT(uploadToBoard()));
    
    deleteBoardAct=new QAction(tr("Clean the Board"), 0, this);
    deleteBoardAct->setStatusTip(tr("removes the recently uploaded program"));
    connect(deleteBoardAct, SIGNAL(activated()), this, SLOT(deleteBoard()));
}

void MainWindow::createMenus()
{
    fileMenu = new QPopupMenu(this);
    newAct->addTo(fileMenu);
    openAct->addTo(fileMenu);
    saveAct->addTo(fileMenu);
    saveAsAct->addTo(fileMenu);
    fileMenu->insertSeparator();
    exitAct->addTo(fileMenu);

    editMenu = new QPopupMenu(this);
    cutAct->addTo(editMenu);
    copyAct->addTo(editMenu);
    pasteAct->addTo(editMenu);
    deleteAct->addTo(editMenu);

    windowsMenu = new QPopupMenu(this);
    createWindowsMenu();

    helpMenu = new QPopupMenu(this);
    aboutAct->addTo(helpMenu);
    aboutQtAct->addTo(helpMenu);
 

    projectMenu = new QPopupMenu(this);        	//project menu creation
    compileAct->addTo(projectMenu);             
    debugAct->addTo(projectMenu); 
    simulateAct->addTo(projectMenu);
    uploadAct->addTo(projectMenu);
    deleteBoardAct->addTo(projectMenu);

    menuBar()->insertItem(tr("&File"), fileMenu);
    menuBar()->insertItem(tr("&Edit"), editMenu);
    menuBar()->insertItem(tr("&Windows"), windowsMenu);
    menuBar()->insertSeparator();
    menuBar()->insertItem(tr("&Help"), helpMenu);
    menuBar()->insertItem(tr("&Project"),projectMenu);
    
}

void MainWindow::createToolBars()
{
    fileToolBar = new QToolBar(tr("File"), this);
    newAct->addTo(fileToolBar);
    openAct->addTo(fileToolBar);
    saveAct->addTo(fileToolBar);

    editToolBar = new QToolBar(tr("Edit"), this);
    cutAct->addTo(editToolBar);
    copyAct->addTo(editToolBar);
    pasteAct->addTo(editToolBar);
}

void MainWindow::createStatusBar()
{
    readyLabel = new QLabel(tr(" Ready"), this);

    modLabel = new QLabel(tr(" MOD "), this);
    modLabel->setAlignment(AlignHCenter);
    modLabel->setMinimumSize(modLabel->sizeHint());
    modLabel->clear();

    statusBar()->addWidget(readyLabel, 1);
    statusBar()->addWidget(modLabel);
}

void MainWindow::newFile()
{
    Editor *editor = createEditor();
    editor->newFile();
    editor->show();
}

Editor *MainWindow::createEditor()
{
    Editor *editor = new Editor(workspace);
    connect(editor, SIGNAL(copyAvailable(bool)),
            this, SLOT(copyAvailable(bool)));
    connect(editor, SIGNAL(modificationChanged(bool)),
            this, SLOT(updateModIndicator()));
    return editor;
}

void MainWindow::open()
{
    Editor *editor = createEditor();
    if (editor->open())
        editor->show();
    else
        editor->close();
}

void MainWindow::openFile(const QString &fileName)
{
    Editor *editor = createEditor();
    if (editor->openFile(fileName))
        editor->show();
    else
        editor->close();
}

void MainWindow::save()
{
    if (activeEditor()) {
        activeEditor()->save();
        updateModIndicator();
    }
}




void MainWindow::compileWindowOpen()
{  
sdcc->clearArguments();

    if (activeEditor())
    {
    curhex=activeEditor()->getcurFile();
    curhex.replace(curhex.length()-1, 1, "hex");
    
    directory = activeEditor()->getcurFile();
        
          if (!directory) {compmess->setText("No file to compile"); return;}
       
       if (directory.at(directory.length()-1)!='c') {compmess->setText("No c source file");
	   compmess->show(); return;}       
    directory.remove(directory.length() - activeEditor()->strippedName(directory).length(),activeEditor()->strippedName(directory).length()+1);
        sdcc->addArgument( "sdcc");
        sdcc->addArgument( "-mpic14" );
        sdcc->addArgument( "-p16f877" );
        sdcc->addArgument("--debug");
        sdcc->addArgument("-o");
        sdcc->addArgument(directory);
        sdcc->addArgument( activeEditor()->getcurFile() );
        connect( sdcc, SIGNAL(readyReadStdout()), this, SLOT(readfromstdout()));
	connect( sdcc, SIGNAL(readyReadStderr()), this, SLOT(readfromstderr()));           
        connect(sdcc, SIGNAL(processExited()), this, SLOT(compileWindowClose()));
	if (!sdcc->start()) 
	{compmess->setText("compilation couldn't start");
	    compmess->show(); return;}
	
    }
    
    else { compmess->setText("No file to compile"); compmess->show();return;}
}


void MainWindow::readfromstdout()
{
    compmess->setText("compilation status " + sdcc->readStdout());	
    compmess->show();
}

void MainWindow::readfromstderr()
{
    compmess->setText( sdcc->readStderr());	
    compmess->show();
}

void MainWindow::readfromstdoutodyssey()
{
    dellmess->setText("clean status:" + odysseyrem->readStdout());
    dellmess->show();    
}

void MainWindow::readfromstdoutodysseyupload()
{
    upmess->setText("upload status:" + odysseywri->readStdout());
    upmess->show();    
}


void MainWindow::compileWindowClose()
{
          int status = compmess->exec();
       if (status==QMessageBox::Accepted)  compmess->hide();
}	

void MainWindow::deleteBoardClose()
{
  int status = dellmess->exec();
  if (status==QMessageBox::Accepted) dellmess->hide();
}

void MainWindow::deleteBoard()
{ 
    odysseyrem->addArgument("odyssey");
    odysseyrem->addArgument("PIC16F877");
    odysseyrem->addArgument("erase");
    connect(odysseyrem, SIGNAL(readyReadStdout()), this, SLOT(readfromstdoutodyssey()));
    connect(odysseyrem, SIGNAL(processExited()), this, SLOT(deleteBoardClose()));   
    if (!odysseyrem->start()) {dellmess->setText("Erase failed\n"); 
	dellmess->show();}
 }   
     
void MainWindow::uploadToBoard()
{    
    QString hexfile;  
    odysseywri->addArgument("odyssey");
    odysseywri->addArgument("PIC16F877");
    odysseywri->addArgument("write");
    hexfile=activeEditor()->getcurFile();
    hexfile.replace(hexfile.length()-1, 1, "hex"); 
    odysseywri->addArgument(hexfile);

     connect(odysseywri, SIGNAL(readyReadStdout()), this, SLOT(readfromstdoutodysseyupload()));
     connect(odysseywri, SIGNAL(processExited()), this, SLOT(uploadToBoardClose()));
    if (!odysseywri->start()) {upmess->setText("Upload Failed\n");
	upmess->show();}
}

void MainWindow::uploadToBoardClose()
{
  int status = upmess->exec();
 if (status==QMessageBox::Accepted) upmess->hide();
}

void MainWindow::simulateWindowOpen()
{
    	simulator *dialog = new simulator(this);
	QWidget app(dialog);
	dialog->show();
	 app.setEnabled(true);
	 app.show();
	
}

void MainWindow::debugWindowOpen()
{  
        watchwindow *debugger = new watchwindow(this);
	QWidget dapp(debugger);
	debugger->show();
	dapp.setEnabled(true);
	dapp.show();
 /*   bool debugging;
    debugger *dialog = new debugger(this);
    QWidget app(dialog);
	
    debug->addArgument("sdcdb");
    directory=activeEditor()->getcurFile().remove(activeEditor()->getcurFile().length()-2,2);
    debug->addArgument(directory);
    debugging=debug->start();
    if(!debugging) debugmess->show();
    else { 
	dialog->show();
	app.setEnabled(true);
	app.show();
    } */
}	

Editor *MainWindow::activeEditor()
{
    return (Editor *)workspace->activeWindow();
}

void MainWindow::saveAs()
{
    if (activeEditor()) {
        activeEditor()->saveAs();
        updateModIndicator();
    }
}

void MainWindow::cut()
{
    if (activeEditor())
        activeEditor()->cut();
}

void MainWindow::copy()
{
    if (activeEditor())
        activeEditor()->copy();
}

void MainWindow::paste()
{
    if (activeEditor())
        activeEditor()->paste();
}

void MainWindow::del()
{
    if (activeEditor())
        activeEditor()->del();
}

void MainWindow::about()
{
    QMessageBox::about(this, tr("About PicSim"),
            tr("<h2>PicSim</h2>"
               "<p>Copyright &copy; 2007 SoSoft"
               "<p>PicSim is a PicDev Board development environment "));
}

void MainWindow::updateMenus()
{
    bool hasEditor = (activeEditor() != 0);
    saveAct->setEnabled(hasEditor);
    saveAsAct->setEnabled(hasEditor);
    pasteAct->setEnabled(hasEditor);
    deleteAct->setEnabled(hasEditor);
    copyAvailable(activeEditor()
                  && activeEditor()->hasSelectedText());
    closeAct->setEnabled(hasEditor);
    closeAllAct->setEnabled(hasEditor);
    tileAct->setEnabled(hasEditor);
    cascadeAct->setEnabled(hasEditor);
    nextAct->setEnabled(hasEditor);
    previousAct->setEnabled(hasEditor);

    windowsMenu->clear();
    createWindowsMenu();
}

void MainWindow::createWindowsMenu()
{
    closeAct->addTo(windowsMenu);
    closeAllAct->addTo(windowsMenu);
    windowsMenu->insertSeparator();
    tileAct->addTo(windowsMenu);
    cascadeAct->addTo(windowsMenu);
    windowsMenu->insertSeparator();
    nextAct->addTo(windowsMenu);
    previousAct->addTo(windowsMenu);

    if (activeEditor()) {
        windowsMenu->insertSeparator();
        windows = workspace->windowList();
        int numVisibleEditors = 0;

        for (int i = 0; i < (int)windows.count(); ++i) {
            QWidget *win = windows.at(i);
            if (!win->isHidden()) {
                QString text = tr("%1 %2")
                               .arg(numVisibleEditors + 1)
                               .arg(win->caption());
                if (numVisibleEditors < 9)
                    text.prepend("&");
                int id = windowsMenu->insertItem(
                              text, this, SLOT(activateWindow(int)));
                bool isActive = (activeEditor() == win);
                windowsMenu->setItemChecked(id, isActive);
                windowsMenu->setItemParameter(id, i);
                ++numVisibleEditors;
            }
        }
    }
}

void MainWindow::activateWindow(int param)
{
    QWidget *win = windows.at(param);
    win->show();
    win->setFocus();
}

void MainWindow::copyAvailable(bool available)
{
    cutAct->setEnabled(available);
    copyAct->setEnabled(available);
}

void MainWindow::updateModIndicator()
{
    if (activeEditor() && activeEditor()->isModified())
        modLabel->setText(tr("MOD"));
    else
        modLabel->clear();
}

void MainWindow::closeEvent(QCloseEvent *event)
{
    workspace->closeAllWindows();
    if (activeEditor())
        event->ignore();
    else
        event->accept();
}

